package kiyut.sketsa.modules.emoticonsymbol;

import kiyut.sketsa.api.symbollibrary.AbstractSymbol;


/** The base class for Emoticon Symbol.
 * This class only set the category to Emoticon
 *
 * @author Kiyut
 */
public abstract class EmoticonSymbol extends AbstractSymbol {
    
    /** 
     * This method returns Emoticon as String.
     * {@inheritDoc} 
     */
    public String getCategory() {
        return "Emoticon";
    }
    
}
